/* This takes care of all memory requests, and frees
 * it indirects malloc so that lame machines which have 16 bit integers may
 * call to request larger amounts of memory than 64k
 * ### this may need modification when it is ported to a new machine
 */

#include	"includes.h"

static ULONG
	numAllocated=0;

static char errorFamily[]="xMemory";

enum
	{
	NOMEMORY
	};

static char *errorMembers[]=
	{
	"NoMemory"
	};

static char *errorDescriptions[]=
	{
	"Insufficient memory"
	};

void *MNewPtr(ULONG theSize)
/* Call this to get a contiguous, aligned pointer to memory
 * If NULL is returned, the request could not be granted
 * If NULL is returned, GetError may be called to find out what went wrong
 */
{
void
	*thePointer;

	if( (thePointer=(void *)malloc((int)theSize)) )
		{
		numAllocated++;
		}
	else
		{
		SetError(errorFamily,errorMembers[NOMEMORY],errorDescriptions[NOMEMORY]);	/* could not allocate memory, so tell user */
		}
	return(thePointer);
}

void *MNewPtrClr(ULONG theSize)
/* Call this to get a contiguous, aligned pointer to cleared memory
 * If NULL is returned, the request could not be granted
 * If NULL is returned, GetError may be called to find out what went wrong
 */
{
void
	*thePointer;

	if( (thePointer=(void *)calloc((int)theSize,1)) )
		{
		numAllocated++;
		}
	else
		{
		SetError(errorFamily,errorMembers[NOMEMORY],errorDescriptions[NOMEMORY]);	/* could not allocate memory, so tell user */
		}
	return(thePointer);
}

void MDisposePtr(void *thePointer)
/* Dispose of memory allocated by MNewPtr
 */
{
	if(thePointer)
		{
		numAllocated--;
		free(thePointer);
		}
}

void MFillMem(void *theBuffer,ULONG numBytes,UBYTE theValue)
/* Fill memory pointed to by theBuffer for numBytes with theValue
 */
{
int
	numToSet;
ULONG
	setIndex;

	setIndex=0;
	while(numBytes)
		{
		numToSet=Min(numBytes,INT_MAX);		/* get number of bytes to set this time */
		memset(&(((UBYTE *)theBuffer)[setIndex]),theValue,numToSet);
		setIndex+=(ULONG)numToSet;			/* move to next location */
		numBytes-=(ULONG)numToSet;			/* this many less bytes to go */
		}
}

void MMoveMem(void *source,void *dest,ULONG numBytes)
/* Move memory from source to dest for numBytes
 * This must be smart enough to prevent overlapping areas from causing trouble
 */
{
int
	numToMove;
ULONG
	moveIndex;

	moveIndex=0;
	while(numBytes)
		{
		numToMove=Min(numBytes,INT_MAX);	/* dont pass an int that is too big */
		bcopy(&(((UBYTE *)source)[moveIndex]),&(((UBYTE *)dest)[moveIndex]),numToMove);
		moveIndex+=(ULONG)numToMove;
		numBytes-=(ULONG)numToMove;			/* this many less bytes to go */
		}
}

ULONG MGetNumAllocatedPointers()
/* Return the current number of allocated pointers
 */
{
	return(numAllocated);
}
